/////////////////////////////////////////////////////////////////////////////
//
//   File:		Pose2D.cc
//
//   Author(s):		Ioannis Rekleitis
//
//   Created:		16 Sep 2003
//
//   Last Revision:	Sept. 16, 2003
//
//   Description:	Defines the necessary pose functionality 
//
//   Copyright (c) 2003 by Ioannis Rekleitis, Carnegie Mellon University.
//   Please see the copyright notice included in this distribution for 
//   full details.
//
////////////////////////////////////////////////////////////////////////////

#define POSE2D_C

#include "Pose2D.h"

const double Pose2D::mPI = 3.14159265;
const double Pose2D::Deg2Rad = 0.0174532925; 
const double Pose2D::Rad2Deg = 57.2957795786;

// copy constructor, ensure the data is copied 
Pose2D::Pose2D(const Pose2D &rhs) { 
  x = rhs.x;
  y = rhs.y;
  theta=rhs.theta;
}

bool Pose2D::operator==(const Pose2D &rhs) const {
  if (x != rhs.x) 
    return false;
  if (y != rhs.y)
    return false;
  if (theta != rhs.theta)
    return false;

  return true;
}

bool Pose2D::operator!=(const Pose2D &rhs) const {
  if (x != rhs.x) 
    return true;
  if (y != rhs.y)
    return true;
  if (theta != rhs.theta)
    return true;

  return false;
}

 // ensure the data is copied
Pose2D &Pose2D::operator=(const Pose2D &rhs) {
  if (this == &rhs)
    return *this;
  x = rhs.x;
  y = rhs.y;
  theta=rhs.theta;
  return *this;
}

ostream &operator<<(ostream &os, const Pose2D &p) {
  os << "(" << p.x << ", " << p.y << ", "<<p.r2d(p.theta)<<")";
  return os;
}

istream &operator>>(istream &is, Pose2D &p) {
  double px, py, pt;
//   const double Deg2Rad = 0.0174532925; 

  is >> px >> py >> pt;
  p.x = px;
  p.y = py;
  p.theta =p.d2r(p.theta0360(pt));
  cout<<"p.theta "<<p.theta<<endl;

  return is;
}

// returns the distance between current point and point q
double Pose2D::distance(const Pose2D q) const {
  double dx = x - q.x;
  double dy = y - q.y;

  return sqrt(dx*dx + dy*dy);
}

double Pose2D::angleDiffDeg(double B)  {
//This function operates in degrees.
cerr<<"Error Pose2D::angleDiff Not Done Yet!"<<endl;
return(-1);
  if((theta>90.0) && (theta<270.0)) {
    B=theta0360(B);
    return(theta180(B-theta));
  }
  else {
    B=theta180(B);
    return(theta180(B-theta180(theta)));
  }
}

double Pose2D::angleDiff(Pose2D B)  {
//This function operates in degrees.
cerr<<"Error Pose2D::angleDiff Not Done Yet!"<<endl;
return(-1);

  if((theta>90.0) && (theta<270.0)) {
    return(theta180(B.orientation()-theta));
  }
  else {
    double b=theta180(B.orientation());
    return(theta180(b-theta180(theta)));
  }
}

///////////////////////// Private Functions ///////////////////
double  
Pose2D::theta02p(double t) {
  double doublePI=2.0*mPI;
  while (t<0) t+=doublePI;
  while (t>=doublePI) t-=doublePI;
  return(t);
}

double  
Pose2D::thetapp(double t) {
//   const double mPI = 3.14159265;
  double doublePI=2.0*mPI;
  while (t<-mPI) t+=doublePI;
  while (t>=mPI) t-=doublePI;
  return(t);
}
double  
Pose2D::theta0360(double t) {
// Operates in degrees.
  while (t<0) t+=360.0;
  while (t>=360.0) t-=360.0;
  return(t);
}

double  
Pose2D::theta180(double t) {
// Operates in degrees.
  while (t<-180.0) t+=360.0;
  while (t>=180.0) t-=360.0;
  return(t);
}

/////////////////////////////////////////////////////////////////////////////
// EOF
